
///////////////////////////////////////////////////////////////////////////
// This program is free software: you can redistribute it and/or modify  //
// it under the terms of the version 3 of the GNU General Public License //
// as published by the Free Software Foundation.                         //
//                                                                       //
// This program is distributed in the hope that it will be useful, but   //
// WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      //
// General Public License for more details.                              //
//                                                                       //
// You should have received a copy of the GNU General Public License     //
// along with this program. If not, see <http://www.gnu.org/licenses/>.  //
//                                                                       //
// Written by Francois Fleuret                                           //
// (C) Idiap Research Institute                                          //
//                                                                       //
// Contact <francois.fleuret@idiap.ch> for comments & bug reports        //
///////////////////////////////////////////////////////////////////////////

#ifndef LABELLED_IMAGE_H
#define LABELLED_IMAGE_H

#include "rich_image.h"
#include "pose.h"
#include "pose_cell.h"

class LabelledImage : public RichImage {
  int _nb_targets;
  Pose *_target_poses;

  static const int file_format_version = 2;

public:

  LabelledImage();
  LabelledImage(int width, int height, int nb_targets);

  virtual ~LabelledImage();

  inline int nb_targets() { return _nb_targets; }
  inline Pose *get_target_pose(int n) { return _target_poses + n; }

  // The label of a cell can be +1 if it contains a target and -1 if
  // it is far enough from any target
  int pose_cell_label(PoseCell *cell);

  virtual void write(ostream *out);
  virtual void read(istream *in);
};

#endif
