
///////////////////////////////////////////////////////////////////////////
// This program is free software: you can redistribute it and/or modify  //
// it under the terms of the version 3 of the GNU General Public License //
// as published by the Free Software Foundation.                         //
//                                                                       //
// This program is distributed in the hope that it will be useful, but   //
// WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      //
// General Public License for more details.                              //
//                                                                       //
// You should have received a copy of the GNU General Public License     //
// along with this program. If not, see <http://www.gnu.org/licenses/>.  //
//                                                                       //
// Written by Francois Fleuret                                           //
// (C) Idiap Research Institute                                          //
//                                                                       //
// Contact <francois.fleuret@idiap.ch> for comments & bug reports        //
///////////////////////////////////////////////////////////////////////////

#include "pose_cell_set.h"
#include "global.h"

PoseCellSet::PoseCellSet() {
  _nb_added = 0;
  _max_nb = max_nb_cells;
  _cells = new PoseCell[_max_nb];
}

PoseCellSet::~PoseCellSet() {
  delete[] _cells;
}

void PoseCellSet::erase_content() {
  _nb_added = 0;
}

PoseCell *PoseCellSet::get_containing_cell(Pose *p) {
  for(int c = 0; c < _nb_added; c++) {
    if(_cells[c].contains(p)) return _cells + c;
  }
  return 0;
}

void PoseCellSet::add_cell(PoseCell *cell) {
  if(_nb_added < _max_nb) {
    _cells[_nb_added] = *cell;
    _nb_added++;
  } else {
    cerr << "Too many pose cells!" << endl;
    abort();
  }
}
